@@ -2,6 +2,8 @@
 ######################################################################################################
 ######################################################################################################
 ######################################################################################################
+
+
 ### decomposition of a square  matrix
 MatDecomp <- function(mtx, method) {
   # Controlli preliminari veloci
@@ -60,6 +62,33 @@ MatLogDet <- function(mat.decomp, method) {
     NA
   }
 }
+
+# utility function for geokrig
+getInvC <- function(covmatrix, CC, mse = TRUE) {
+    if (!covmatrix$sparse) {
+      U <- tryCatch({
+        FastGP::rcppeigen_get_chol(covmatrix$covmatrix)
+      }, error = function(e) {
+        stop("Covariance matrix is not positive definite")
+      })
+      vec <- forwardsolve(U, CC)
+      Invc <- forwardsolve(U, vec, transpose = TRUE)
+      mse_val <- if (mse) as.numeric(crossprod(vec)) else NULL
+    } else {
+      cc <- if (spam::is.spam(covmatrix$covmatrix)) covmatrix$covmatrix else spam::as.spam(covmatrix$covmatrix)
+      U <- tryCatch({
+        spam::chol.spam(cc)
+      }, error = function(e) {
+        stop("Covariance matrix is not positive definite")
+      })
+      vec <- spam::forwardsolve(U, CC)
+      Invc <- spam::backsolve(U, vec)
+      mse_val <- if (mse) as.numeric(spam::crossprod.spam(vec)) else NULL
+    }
+    list(a = Invc, b = mse_val)
+  }
+
+  
 ######################################################################################################
 ######################################################################################################
 ######################################################################################################
