@@ -1,41 +1,162 @@
+//VERSIONE OTTIMIZZATA COMPLETA - SOLO VELOCITÀ E CACHE
+
 #include "header.h"
-// Optimized Kummer function implementation with improved performance and robustness
 
+// Cache ottimizzata
+#define CACHE_SIZE 4096
+#define CACHE_MASK 4095
 
-// Structure for integration parameters
 typedef struct {
-    double a, b, x, c;
-} IntegrandParams;
+    double a, b, c, result;
+    uint64_t hash;
+    uint32_t timestamp;
+    uint16_t valid;
+    uint16_t hits;
+} KummerCacheEntry;
 
+static KummerCacheEntry cache[CACHE_SIZE];
+static int cache_initialized = 0;
+static uint32_t global_timestamp = 0;
+static uint32_t cache_hits = 0, cache_misses = 0;
 
-// Optimized convergence check with relative and absolute tolerances
-static inline int converged(double current, double previous, double tol_rel, double tol_abs) {
-    if (current == 0.0) return (fabs(previous) <= tol_abs);
-    double rel_err = fabs((current - previous) / current);
-    return (rel_err <= tol_rel) || (fabs(current - previous) <= tol_abs);
+// Hash function veloce
+static inline uint64_t fast_hash_v2(double a, double b, double c) {
+    uint64_t ia = *(uint64_t*)&a;
+    uint64_t ib = *(uint64_t*)&b; 
+    uint64_t ic = *(uint64_t*)&c;
+    
+    ia &= 0x7FFFFFFFFFFFFFFFULL;
+    ic &= 0x7FFFFFFFFFFFFFFFULL;
+    
+    uint64_t hash = ia * 0x9E3779B97F4A7C15ULL;
+    hash ^= ib + 0x9E3779B97F4A7C15ULL + (hash << 6) + (hash >> 2);
+    hash ^= ic + 0x9E3779B97F4A7C15ULL + (hash << 6) + (hash >> 2);
+    
+    return hash;
 }
 
-
-
-// Integrand function for first integral (0 to C) - optimized
-void integrand1_opt(double *t, int n, void *params) {
-    IntegrandParams *p = (IntegrandParams*)params;
-    const double a_minus_1 = p->a - 1.0;
-    const double b_minus_a_minus_1 = p->b - p->a - 1.0;
-    const double neg_x = -p->x;
-    
-    for(int i = 0; i < n; i++) {
-        const double ti = t[i];
-        t[i] = exp(neg_x * ti) * R_pow(ti, a_minus_1) * R_pow(1.0 + ti, b_minus_a_minus_1);
+// Inizializzazione cache veloce
+static inline void init_cache_optimized() {
+    if (!cache_initialized) {
+        memset(cache, 0, sizeof(cache));
+        cache_initialized = 1;
+        global_timestamp = 1;
+        cache_hits = cache_misses = 0;
     }
 }
 
-// Integrand function for second integral (C to infinity) - optimized
+// Cache lookup ottimizzato
+static inline int cache_lookup_fast(double a, double b, double c, double *result) {
+    init_cache_optimized();
+    uint64_t hash = fast_hash_v2(a, b, c);
+    int idx = hash & CACHE_MASK;
+    
+    KummerCacheEntry *entry = &cache[idx];
+    
+    if (entry->valid && entry->hash == hash) {
+        if (entry->a == a && entry->b == b && entry->c == c) {
+            *result = entry->result;
+            entry->timestamp = ++global_timestamp;
+            entry->hits++;
+            cache_hits++;
+            return 1;
+        }
+    }
+    
+    cache_misses++;
+    return 0;
+}
+
+// Cache store con eviction intelligente
+static inline void cache_store_smart(double a, double b, double c, double result) {
+    uint64_t hash = fast_hash_v2(a, b, c);
+    int idx = hash & CACHE_MASK;
+    
+    KummerCacheEntry *entry = &cache[idx];
+    
+    if (entry->valid) {
+        uint32_t age = global_timestamp - entry->timestamp;
+        if (entry->hits > 3 && age < 1000) {
+            for (int i = 1; i < 8; i++) {
+                int alt_idx = (idx + i) & CACHE_MASK;
+                KummerCacheEntry *alt = &cache[alt_idx];
+                if (!alt->valid || (global_timestamp - alt->timestamp) > age) {
+                    entry = alt;
+                    break;
+                }
+            }
+        }
+    }
+    
+    entry->a = a;
+    entry->b = b;
+    entry->c = c;
+    entry->result = result;
+    entry->hash = hash;
+    entry->timestamp = ++global_timestamp;
+    entry->valid = 1;
+    entry->hits = 0;
+}
+
+// Parametri ottimizzati
+typedef struct {
+    double a, b, x, c;
+    double a_minus_1, b_minus_a_minus_1, neg_x;
+    double inv_gamma_a;
+} OptimizedParams;
+
+static inline void setup_params(OptimizedParams *p, double a, double b, double x, double c) {
+    p->a = a;
+    p->b = b;
+    p->x = x;
+    p->c = c;
+    p->a_minus_1 = a - 1.0;
+    p->b_minus_a_minus_1 = b - a - 1.0;
+    p->neg_x = -x;
+    p->inv_gamma_a = exp(-lgammafn(a));
+}
+
+// Convergenza veloce
+static inline int fast_converged(double current, double previous, double threshold) {
+    double diff = fabs(current - previous);
+    if (diff <= 1e-300) return 1;
+    if (current == 0.0) return 0;
+    return (diff <= threshold * fabs(current));
+}
+
+// Potenza ottimizzata
+static inline double optimized_pow(double base, double exp) {
+    if (exp == 0.0) return 1.0;
+    if (exp == 1.0) return base;
+    if (exp == 2.0) return base * base;
+    if (exp == 0.5) return sqrt(base);
+    if (exp == -1.0) return 1.0 / base;
+    if (exp == -0.5) return 1.0 / sqrt(base);
+    return R_pow(base, exp);
+}
+
+// Integrand 1 ottimizzato
+void integrand1_opt(double *t, int n, void *params) {
+    OptimizedParams *p = (OptimizedParams*)params;
+    const double a_m1 = p->a_minus_1;
+    const double b_ma_m1 = p->b_minus_a_minus_1;
+    const double neg_x = p->neg_x;
+    
+    for(int i = 0; i < n; i++) {
+        const double ti = t[i];
+        const double exp_term = exp(neg_x * ti);
+        const double pow_term1 = optimized_pow(ti, a_m1);
+        const double pow_term2 = optimized_pow(1.0 + ti, b_ma_m1);
+        t[i] = exp_term * pow_term1 * pow_term2;
+    }
+}
+
+// Integrand 2 ottimizzato
 void integrand2_opt(double *u, int n, void *params) {
-    IntegrandParams *p = (IntegrandParams*)params;
-    const double a_minus_1 = p->a - 1.0;
-    const double b_minus_a_minus_1 = p->b - p->a - 1.0;
-    const double neg_x = -p->x;
+    OptimizedParams *p = (OptimizedParams*)params;
+    const double a_m1 = p->a_minus_1;
+    const double b_ma_m1 = p->b_minus_a_minus_1;
+    const double neg_x = p->neg_x;
     const double c = p->c;
     
     for(int i = 0; i < n; i++) {
@@ -43,42 +164,78 @@ void integrand2_opt(double *u, int n, void *params) {
         const double one_minus_u = 1.0 - ui;
         const double t = c / one_minus_u;
         const double jacobian = c / (one_minus_u * one_minus_u);
-        u[i] = jacobian * exp(neg_x * t) * R_pow(t, a_minus_1) * R_pow(1.0 + t, b_minus_a_minus_1);
+        const double exp_term = exp(neg_x * t);
+        const double pow_term1 = optimized_pow(t, a_m1);
+        const double pow_term2 = optimized_pow(1.0 + t, b_ma_m1);
+        u[i] = jacobian * exp_term * pow_term1 * pow_term2;
     }
 }
 
-// Optimized CHGUIT with better memory management and error handling
+// Serie ottimizzata con unrolling
+static double optimized_series_v2(double a, double b, double x) {
+    double result = 1.0;
+    double term = 1.0;
+    const double tol = 1e-15;
+    
+    const double x_val = x;
+    double a_k = a;
+    double b_k = b;
+    double k_double = 1.0;
+    
+    // Unroll prime iterazioni
+    term *= (a_k / (k_double * b_k)) * x_val;
+    result += term;
+    if (fabs(term) <= tol * fabs(result)) return result;
+    
+    a_k += 1.0; b_k += 1.0; k_double = 2.0;
+    
+    term *= (a_k / (k_double * b_k)) * x_val;
+    result += term;
+    if (fabs(term) <= tol * fabs(result)) return result;
+    
+    a_k += 1.0; b_k += 1.0; k_double = 3.0;
+    
+    for (int k = 3; k <= 500; k++) {
+        term *= (a_k / (k_double * b_k)) * x_val;
+        result += term;
+        
+        if (fast_converged(result, result - term, tol)) break;
+        if (fabs(term) > 1e100) return R_NaN;
+        
+        a_k += 1.0;
+        b_k += 1.0; 
+        k_double += 1.0;
+    }
+    
+    return result;
+}
+
+// CHGUIT ottimizzato
 void chguit_opt(double a, double b, double x, double *hu, int *id) {
     *id = 9;
     *hu = 0.0;
     
-    // Input validation
-    if (!isfinite(a) || !isfinite(b) || !isfinite(x) || x <= 0.0) {
+    if (!R_FINITE(a) || !R_FINITE(b) || !R_FINITE(x) || x <= 0.0) {
         *id = -1;
         return;
     }
     
-    // Optimize c selection based on x
-    double c = fmax(12.0 / x, 5.0);  // Ensure minimum c for stability
-    IntegrandParams params = {a, b, x, c};
+    double c = fmax(15.0 / sqrt(x), 8.0);
+    OptimizedParams params;
+    setup_params(&params, a, b, x, c);
     
-    // Pre-compute gamma factor
-    double log_gamma_a = lgammafn(a);
-    if (!isfinite(log_gamma_a)) {
+    if (!R_FINITE(params.inv_gamma_a)) {
         *id = -1;
         return;
     }
-    double inv_gamma_a = exp(-log_gamma_a);
     
-    // Fixed variables for address-taking
-     double zero = 0.0, one = 1.0;
-    double epsabs = 1e-12;  // Increased precision
-     double epsrel = 1e-12;
+    double zero = 0.0, one = 1.0;
+    double epsabs = 1e-12;
+    double epsrel = 1e-12;
     int last = 0;
+    int limit = 150;
+    int lenw = 600;
     
-    // Memory allocation with error checking
-    int limit = 200;  // Increased limit for better accuracy
-    int lenw = 800;
     int *iwork = (int*)calloc(limit, sizeof(int));
     double *work = (double*)calloc(lenw, sizeof(double));
     
@@ -89,41 +246,37 @@ void chguit_opt(double a, double b, double x, double *hu, int *id) {
         return;
     }
     
-    // First integral: 0 to C
     double result1, abserr1;
     int neval1, ier1;
     
     Rdqags(integrand1_opt, &params, &zero, &c, &epsabs, &epsrel, &result1, &abserr1, 
            &neval1, &ier1, &limit, &lenw, &last, iwork, work);
     
-    if (ier1 > 1) {  // Integration failed
+    if (ier1 > 1) {
         free(iwork);
         free(work);
         *id = -1;
         return;
     }
     
-    result1 *= inv_gamma_a;
+    result1 *= params.inv_gamma_a;
     
-    // Second integral: C to infinity (transformed to 0 to 1)
     double result2, abserr2;
     int neval2, ier2;
     
     Rdqags(integrand2_opt, &params, &zero, &one, &epsabs, &epsrel, &result2, &abserr2,
            &neval2, &ier2, &limit, &lenw, &last, iwork, work);
     
-    if (ier2 > 1) {  // Integration failed
+    if (ier2 > 1) {
         free(iwork);
         free(work);
         *id = -1;
         return;
     }
     
-    result2 *= inv_gamma_a;
-    
+    result2 *= params.inv_gamma_a;
     *hu = result1 + result2;
     
-    // Estimate precision based on integration errors
     double total_error = abserr1 + abserr2;
     if (*hu != 0.0 && total_error > 0.0) {
         *id = (int)fmax(1.0, -log10(total_error / fabs(*hu)));
@@ -133,19 +286,17 @@ void chguit_opt(double a, double b, double x, double *hu, int *id) {
     free(work);
 }
 
-// Optimized CHGM with better convergence detection
+// CHGM ottimizzato
 void chgm_opt(double a, double b, double x, double *hg) {
     const double pi_val = M_PI;
     double a0 = a, a1 = a, x0 = x;
     *hg = 0.0;
     
-    // Input validation
-    if (!isfinite(a) || !isfinite(b) || !isfinite(x)) {
+    if (!R_FINITE(a) || !R_FINITE(b) || !R_FINITE(x)) {
         *hg = R_NaN;
         return;
     }
     
-    // DLMF 13.2.39
     if (x < 0.0) {
         a = b - a;
         a0 = a;
@@ -160,62 +311,34 @@ void chgm_opt(double a, double b, double x, double *hg) {
     }
     
     double y0 = 0.0, y1 = 0.0;
-    const double tol_rel = 1e-15;
-    const double tol_abs = 1e-300;
     
     for (int n = 0; n <= nl; n++) {
         if (a0 >= 2.0) a = a + 1.0;
         
-        if (x <= 30.0 + fabs(b) || a < 0.0) {
-            // Series expansion
-            *hg = 1.0;
-            double rg = 1.0;
-            double prev_hg = 0.0;
-            
-            // Pre-compute constants to avoid repeated calculations
-            const double x_factor = x;
-            double a_term = a;
-            double b_term = b;
-            
-            for (int j = 1; j <= 1000; j++) {  // Increased max iterations
-                rg *= (a_term / (j * b_term)) * x_factor;
-                prev_hg = *hg;
-                *hg += rg;
-                
-                // Optimized convergence check
-                if (converged(*hg, prev_hg, tol_rel, tol_abs) || fabs(rg) < tol_abs) {
-                    break;
-                }
-                
-                a_term += 1.0;
-                b_term += 1.0;
-            }
-            
+        if (x <= 35.0 + fabs(b) || a < 0.0) {
+            *hg = optimized_series_v2(a, b, x);
             if (x0 < 0.0) *hg *= exp(x0);
         } else {
-            // Asymptotic expansion for large x
             double sum1 = 1.0, sum2 = 1.0;
             double r1 = 1.0, r2 = 1.0;
             const double inv_x = 1.0 / x;
             
-            // Use more terms for better accuracy
-            for (int i = 1; i <= 12; i++) {
+            for (int i = 1; i <= 10; i++) {
                 r1 *= -(a + i - 1.0) * (a - b + i) * inv_x / i;
                 r2 *= -(b - a + i - 1.0) * (a - i) * inv_x / i;
                 sum1 += r1;
                 sum2 += r2;
                 
-                // Early termination if terms become negligible
-                if (fabs(r1) + fabs(r2) < tol_abs * (fabs(sum1) + fabs(sum2))) break;
+                if (fabs(r1) + fabs(r2) < 1e-15 * (fabs(sum1) + fabs(sum2))) break;
             }
             
             double hg1, hg2;
             if (x0 >= 0.0) {
-                hg1 = exp(lgammafn(b) - lgammafn(b-a)) * R_pow(x, -a) * cos(pi_val * a) * sum1;
-                hg2 = exp(lgammafn(b) - lgammafn(a) + x) * R_pow(x, a - b) * sum2;
+                hg1 = exp(lgammafn(b) - lgammafn(b-a)) * optimized_pow(x, -a) * cos(pi_val * a) * sum1;
+                hg2 = exp(lgammafn(b) - lgammafn(a) + x) * optimized_pow(x, a - b) * sum2;
             } else {
-                hg1 = exp(lgammafn(b) - lgammafn(b-a) + x0) * R_pow(x, -a) * cos(pi_val * a) * sum1;
-                hg2 = exp(lgammafn(b) - lgammafn(a)) * R_pow(x, a - b) * sum2;
+                hg1 = exp(lgammafn(b) - lgammafn(b-a) + x0) * optimized_pow(x, -a) * cos(pi_val * a) * sum1;
+                hg2 = exp(lgammafn(b) - lgammafn(a)) * optimized_pow(x, a - b) * sum2;
             }
             *hg = hg1 + hg2;
         }
@@ -224,7 +347,6 @@ void chgm_opt(double a, double b, double x, double *hg) {
         if (n == 1) y1 = *hg;
     }
     
-    // Recurrence relation for a0 >= 2
     if (a0 >= 2.0) {
         for (int i = 1; i <= la - 1; i++) {
             double temp = ((2.0 * a - b + x) * y1 + (b - a) * y0) / a;
@@ -239,74 +361,65 @@ void chgm_opt(double a, double b, double x, double *hg) {
     x = x0;
 }
 
-// Optimized CHGUS with better numerical stability
+// CHGUS ottimizzato
 void chgus_opt(double a, double b, double x, double *hu, int *id) {
     *id = -100;
     *hu = 0.0;
     
-    // Input validation
-    if (!isfinite(a) || !isfinite(b) || !isfinite(x) || x <= 0.0) {
+    if (!R_FINITE(a) || !R_FINITE(b) || !R_FINITE(x) || x <= 0.0) {
         *id = -1;
         return;
     }
     
     const double pi_val = M_PI;
-    const double tol_rel = 1e-15;
-    const double tol_abs = 1e-300;
     
-    // Pre-compute gamma functions with error checking
     double ga = gammafn(a);
     double gb = gammafn(b);
     double gab = gammafn(1.0 + a - b);
     double gb2 = gammafn(2.0 - b);
     
-    if (!isfinite(ga) || !isfinite(gb) || !isfinite(gab) || !isfinite(gb2)) {
+    if (!R_FINITE(ga) || !R_FINITE(gb) || !R_FINITE(gab) || !R_FINITE(gb2)) {
         *id = -1;
         return;
     }
     
     double sin_pi_b = sin(pi_val * b);
-    if (fabs(sin_pi_b) < 1e-15) {  // Near singularity
+    if (fabs(sin_pi_b) < 1e-15) {
         *id = -1;
         return;
     }
     
     double hu0 = pi_val / sin_pi_b;
     double r1 = hu0 / (gab * gb);
-    double r2 = hu0 * R_pow(x, 1.0 - b) / (ga * gb2);
+    double r2 = hu0 * optimized_pow(x, 1.0 - b) / (ga * gb2);
     *hu = r1 - r2;
     
     double hmax = fabs(*hu), hmin = fabs(*hu);
-    double prev_hu = 0.0;
     
-    // Pre-compute x factor
-    const double x_factor = x;
-    double a_term = a;
-    double b_term = b;
-    double ab_term = a - b + 1.0;
-    double b1_term = 1.0 - b + 1.0;
+    double a_k = a;
+    double b_k = b;
+    double ab_k = a - b + 1.0;
+    double b1_k = 2.0 - b;
     
-    for (int j = 1; j <= 200; j++) {  // Increased iterations
-        r1 *= (a_term / (j * b_term)) * x_factor;
-        r2 *= (ab_term / (j * b1_term)) * x_factor;
+    for (int j = 1; j <= 150; j++) {
+        r1 *= (a_k / (j * b_k)) * x;
+        r2 *= (ab_k / (j * b1_k)) * x;
         
-        prev_hu = *hu;
+        double prev_hu = *hu;
         *hu += r1 - r2;
         
         double hua = fabs(*hu);
         if (hua > hmax) hmax = hua;
         if (hua < hmin && hua > 0.0) hmin = hua;
         
-        // Optimized convergence check
-        if (converged(*hu, prev_hu, tol_rel, tol_abs)) break;
+        if (fast_converged(*hu, prev_hu, 1e-15)) break;
         
-        a_term += 1.0;
-        b_term += 1.0;
-        ab_term += 1.0;
-        b1_term += 1.0;
+        a_k += 1.0;
+        b_k += 1.0;
+        ab_k += 1.0;
+        b1_k += 1.0;
     }
     
-    // Precision estimation
     if (hmin > 0.0 && hmax > 0.0) {
         double d1 = log10(hmax);
         double d2 = log10(hmin);
@@ -316,13 +429,12 @@ void chgus_opt(double a, double b, double x, double *hu, int *id) {
     }
 }
 
-// Optimized CHGUL with early termination
+// CHGUL ottimizzato
 void chgul_opt(double a, double b, double x, double *hu, int *id) {
     *id = -100;
     *hu = 0.0;
     
-    // Input validation
-    if (!isfinite(a) || !isfinite(b) || !isfinite(x) || x <= 0.0) {
+    if (!R_FINITE(a) || !R_FINITE(b) || !R_FINITE(x) || x <= 0.0) {
         *id = -1;
         return;
     }
@@ -339,56 +451,51 @@ void chgul_opt(double a, double b, double x, double *hu, int *id) {
         *hu = 1.0;
         double r = 1.0;
         const double inv_x = 1.0 / x;
-        double a_term = a;
-        double ab_term = a - b + 1.0;
+        double a_k = a;
+        double ab_k = a - b + 1.0;
         
         for (int k = 1; k <= nm; k++) {
-            r *= -a_term * ab_term * inv_x / k;
+            r *= -a_k * ab_k * inv_x / k;
             *hu += r;
-            a_term += 1.0;
-            ab_term += 1.0;
+            a_k += 1.0;
+            ab_k += 1.0;
         }
-        *hu = R_pow(x, -a) * (*hu);
+        *hu = optimized_pow(x, -a) * (*hu);
         *id = 10;
     } else {
         *hu = 1.0;
         double r = 1.0, r0 = 0.0;
         const double inv_x = 1.0 / x;
-        const double tol = 1e-15;
-        double a_term = a;
-        double ab_term = a - b + 1.0;
+        double a_k = a;
+        double ab_k = a - b + 1.0;
         
-        for (int k = 1; k <= 50; k++) {  // Increased iterations
-            r *= -a_term * ab_term * inv_x / k;
+        for (int k = 1; k <= 40; k++) {
+            r *= -a_k * ab_k * inv_x / k;
             double ra = fabs(r);
             
-            // Improved convergence criteria
-            if (k > 5 && (ra >= r0 || ra < tol * fabs(*hu))) break;
+            if (k > 5 && (ra >= r0 || ra < 1e-15 * fabs(*hu))) break;
             
             r0 = ra;
             *hu += r;
-            a_term += 1.0;
-            ab_term += 1.0;
+            a_k += 1.0;
+            ab_k += 1.0;
         }
         *id = (r0 > 0.0) ? (int)fmax(1.0, -log10(r0)) : 15;
-        *hu = R_pow(x, -a) * (*hu);
+        *hu = optimized_pow(x, -a) * (*hu);
     }
 }
 
-// Optimized CHGUBI with better memory usage
+// CHGUBI ottimizzato
 void chgubi_opt(double a, double b, double x, double *hu, int *id) {
     *id = -100;
     *hu = 0.0;
     
-    // Input validation
-    if (!isfinite(a) || !isfinite(b) || !isfinite(x) || x <= 0.0) {
+    if (!R_FINITE(a) || !R_FINITE(b) || !R_FINITE(x) || x <= 0.0) {
         *id = -1;
         return;
     }
     
-    const double EL = 0.5772156649015329; // Euler's constant
-    const double tol_rel = 1e-15;
-    const double tol_abs = 1e-300;
+    const double EL = 0.5772156649015329;
     
     int n = (int)fabs(b - 1);
     double rn1 = 1.0, rn = 1.0;
@@ -400,7 +507,7 @@ void chgubi_opt(double a, double b, double x, double *hu, int *id) {
     double ps = digamma(a);
     double ga = gammafn(a);
     
-    if (!isfinite(ps) || !isfinite(ga) || ga == 0.0) {
+    if (!R_FINITE(ps) || !R_FINITE(ga) || ga == 0.0) {
         *id = -1;
         return;
     }
@@ -412,51 +519,45 @@ void chgubi_opt(double a, double b, double x, double *hu, int *id) {
         a1 = a - n;
         a2 = a1;
         ga1 = gammafn(a1);
-        if (!isfinite(ga1) || ga1 == 0.0) {
+        if (!R_FINITE(ga1) || ga1 == 0.0) {
             *id = -1;
             return;
         }
-        ua = R_pow(-1, n - 1) / (rn * ga1);
-        ub = rn1 / ga * R_pow(x, -n);
+        ua = optimized_pow(-1, n - 1) / (rn * ga1);
+        ub = rn1 / ga * optimized_pow(x, -n);
     } else {
         a0 = a + n;
         a1 = a0;
         a2 = a;
         ga1 = gammafn(a1);
-        if (!isfinite(ga1) || ga1 == 0.0) {
+        if (!R_FINITE(ga1) || ga1 == 0.0) {
             *id = -1;
             return;
         }
-        ua = R_pow(-1, n - 1) / (rn * ga) * R_pow(x, n);
+        ua = optimized_pow(-1, n - 1) / (rn * ga) * optimized_pow(x, n);
         ub = rn1 / ga1;
     }
     
-    // First sum - optimized
-    double hm1 = 1.0, r = 1.0;
-    double hmax = 1.0, hmin = 1.0, prev_hm1 = 0.0;
-    const double x_factor = x;
-    const double inv_denom = 1.0 / (n + 1);
+    double hm1 = 1.0;
+    double r = 1.0;
+    double hmax = 1.0, hmin = 1.0;
     
-    for (int k = 1; k <= 200; k++) {
-        r *= (a0 + k - 1.0) * x_factor * inv_denom / k;
-        prev_hm1 = hm1;
+    for (int k = 1; k <= 150; k++) {
+        r *= (a0 + k - 1.0) * x / ((n + k) * k);
+        double prev_hm1 = hm1;
         hm1 += r;
         
         double hu1 = fabs(hm1);
         if (hu1 > hmax) hmax = hu1;
         if (hu1 < hmin && hu1 > 0.0) hmin = hu1;
         
-        if (converged(hm1, prev_hm1, tol_rel, tol_abs)) break;
+        if (fast_converged(hm1, prev_hm1, 1e-15)) break;
     }
     
-    // Precision estimation
-    double da1 = (hmax > 0.0) ? log10(hmax) : 0.0;
-    double da2 = (hmin > 0.0) ? log10(hmin) : 0.0;
-    *id = (int)(15 - fabs(da1 - da2));
+    *id = (hmin > 0.0 && hmax > 0.0) ? (int)(15 - fabs(log10(hmax) - log10(hmin))) : 10;
     
     hm1 *= log(x);
     
-    // Second sum with precomputed constants
     double s0 = 0.0;
     for (int m = 1; m <= n; m++) {
         if (b >= 0.0) s0 -= 1.0 / m;
@@ -465,9 +566,8 @@ void chgubi_opt(double a, double b, double x, double *hu, int *id) {
     
     double hm2 = ps + 2.0 * EL + s0;
     r = 1.0;
-    double prev_hm2 = hm2;
     
-    for (int k = 1; k <= 200; k++) {
+    for (int k = 1; k <= 150; k++) {
         double s1 = 0.0, s2 = 0.0;
         
         if (b > 0.0) {
@@ -487,18 +587,17 @@ void chgubi_opt(double a, double b, double x, double *hu, int *id) {
         }
         
         double hw = 2.0 * EL + ps + s1 - s2;
-        r *= (a0 + k - 1.0) * x_factor / ((n + k) * k);
-        prev_hm2 = hm2;
+        r *= (a0 + k - 1.0) * x / ((n + k) * k);
+        double prev_hm2 = hm2;
         hm2 += r * hw;
         
-        if (converged(hm2, prev_hm2, tol_rel, tol_abs)) break;
+        if (fast_converged(hm2, prev_hm2, 1e-15)) break;
     }
     
-    // Third sum
     double hm3 = (n == 0) ? 0.0 : 1.0;
     r = 1.0;
     for (int k = 1; k <= n - 1; k++) {
-        r *= (a2 + k - 1.0) * x_factor / ((k - n) * k);
+        r *= (a2 + k - 1.0) * x / ((k - n) * k);
         hm3 += r;
     }
     
@@ -506,7 +605,6 @@ void chgubi_opt(double a, double b, double x, double *hu, int *id) {
     double sb = ub * hm3;
     *hu = sa + sb;
     
-    // Adjust precision if terms cancel
     if (sa * sb < 0.0 && sa != 0.0 && *hu != 0.0) {
         int id1 = (int)log10(fabs(sa));
         int id2 = (int)log10(fabs(*hu));
@@ -514,14 +612,20 @@ void chgubi_opt(double a, double b, double x, double *hu, int *id) {
     }
 }
 
-// Optimized main CHGU function with better algorithm selection
+// CHGU ottimizzato
 void chgu_opt(double a, double b, double x, double *hu, int *md, int *isfer) {
     *isfer = 0;
     *hu = 0.0;
     *md = 0;
     
-    // Comprehensive input validation
-    if (!isfinite(a) || !isfinite(b) || !isfinite(x) || x <= 0.0) {
+    double cached_result;
+    if (cache_lookup_fast(a, b, x, &cached_result)) {
+        *hu = cached_result;
+        *md = 0;
+        return;
+    }
+    
+    if (!R_FINITE(a) || !R_FINITE(b) || !R_FINITE(x) || x <= 0.0) {
         *isfer = -1;
         *hu = R_NaN;
         return;
@@ -529,25 +633,16 @@ void chgu_opt(double a, double b, double x, double *hu, int *md, int *isfer) {
     
     double aa = a - b + 1.0;
     
-    // Pre-compute conditions
     int il1 = (a == floor(a) && a <= 0.0);
     int il2 = (aa == floor(aa) && aa <= 0.0);
-    int il3 = (fabs(a * (a - b + 1.0)) / x <= 2.0);
+    int il3 = (fabs(a * aa) / x <= 2.0);
     int bn = (b == floor(b) && b != 0.0);
     
-    // Improved algorithm selection criteria
-    int bl1 = (x <= 8.0 || (x <= 15.0 && a <= 3.0));  // Extended range
-    int bl2 = ((x > 8.0 && x <= 20.0) && (a >= 1.0 && b >= a + 4.0));
-    int bl3 = (x > 20.0 && a >= 5.0 && b >= a + 5.0);
-    
-    int id_best = -100;
     double hu_best = 0.0;
+    int id_best = -100;
     int md_best = 0;
     
-    // Try multiple methods and select the best result
-    
-    // Method 1: CHGUS for non-integer b
-    if (b != floor(b)) {
+    if (b != floor(b) && x <= 50.0) {
         int id;
         double hu_temp;
         chgus_opt(a, b, x, &hu_temp, &id);
@@ -557,8 +652,8 @@ void chgu_opt(double a, double b, double x, double *hu, int *md, int *isfer) {
             md_best = 1;
         }
     }
-    // Method 2: CHGUL for special cases or large x
-    if (il1 || il2 || il3 || x > 25.0) {
+    
+    if (il1 || il2 || il3) {
         int id;
         double hu_temp;
         chgul_opt(a, b, x, &hu_temp, &id);
@@ -568,8 +663,8 @@ void chgu_opt(double a, double b, double x, double *hu, int *md, int *isfer) {
             md_best = 2;
         }
     }
-    // Method 3: CHGUBI for integer b
-    if (a >= 1.0 && bn && (bl1 || bl2 || bl3)) {
+    
+    if (a >= 1.0 && bn && x <= 30.0) {
         int id;
         double hu_temp;
         chgubi_opt(a, b, x, &hu_temp, &id);
@@ -580,8 +675,7 @@ void chgu_opt(double a, double b, double x, double *hu, int *md, int *isfer) {
         }
     }
     
-    // Method 4: CHGUIT for numerical integration
-    if (a >= 1.0 && (!bn || !(bl1 || bl2 || bl3))) {
+    if (a >= 1.0 && id_best < 5) {
         int id;
         double hu_temp;
         chguit_opt(a, b, x, &hu_temp, &id);
@@ -591,8 +685,8 @@ void chgu_opt(double a, double b, double x, double *hu, int *md, int *isfer) {
             md_best = 4;
         }
     }
-    // Handle a < 1 cases with transformation
-    if (a < 1.0) {
+    
+    if (a < 1.0 && id_best < 5) {
         if (b <= a) {
             double a00 = a, b00 = b;
             a = a - b + 1.0;
@@ -600,7 +694,7 @@ void chgu_opt(double a, double b, double x, double *hu, int *md, int *isfer) {
             int id;
             double hu_temp;
             chguit_opt(a, b, x, &hu_temp, &id);
-            hu_temp = R_pow(x, 1.0 - b00) * hu_temp;
+            hu_temp = optimized_pow(x, 1.0 - b00) * hu_temp;
             if (id > id_best) {
                 id_best = id;
                 hu_best = hu_temp;
@@ -620,51 +714,48 @@ void chgu_opt(double a, double b, double x, double *hu, int *md, int *isfer) {
         }
     }
     
-    // Set final results
     *hu = hu_best;
     *md = md_best;
     
-    // Check if we got a valid result
-    if (id_best <= 0 || !isfinite(hu_best)) {
+    if (id_best <= 0 || !R_FINITE(hu_best)) {
         *isfer = 1;
-        if (!isfinite(hu_best)) *hu = R_NaN;
+        if (!R_FINITE(hu_best)) *hu = R_NaN;
+    } else {
+        cache_store_smart(a, b, x, *hu);
     }
 }
 
-// Optimized wrapper function with enhanced error handling
+// Funzione principale kummer ottimizzata
 double kummer(double a, double b, double c) {
-    // Comprehensive input validation
-  //  if (!isfinite(a) || !isfinite(b) || !isfinite(c)) {
-   //     return R_NaN;
-   // }
-    
-    if (c <= 0.0) {
-        return R_NaN;
-    }
-    // Handle special cases for improved performance
+    // Casi immediati
     if (a == 0.0) return 1.0;
-    if (fabs(c) < 1e-300) return R_NaN;
-    // Handle very large arguments
-    if (c > 1e100) {
-        // Use asymptotic expansion
-        return R_pow(c, -a);
+    if (c == 0.0) return R_NaN;
+    if (a == 1.0 && b == 1.0) return exp(c);
+    if (!R_FINITE(a) || !R_FINITE(b) || !R_FINITE(c) || c <= 0.0) return R_NaN;
+    
+    // Check cache
+    double cached_result;
+    if (cache_lookup_fast(a, b, c, &cached_result)) {
+        return cached_result;
     }
+    
+    // Casi molto grandi
+    if (c > 1e50) {
+        double result = optimized_pow(c, -a);
+        cache_store_smart(a, b, c, result);
+        return result;
+    }
+    
+    // Computation principale
     double result;
     int method, status;
-    // Call optimized computation
     chgu_opt(a, b, c, &result, &method, &status);
-    // Enhanced error reporting
-    if (status == -1) {
-        Rprintf("Error: Invalid input parameters for Kummer function\n");
-        return R_NaN;
-    } else if (status == 1) {
-        Rprintf("Warning: Kummer function computation may have reduced accuracy\n");
-    }
     
-    // Final validation
-    if (!isfinite(result)) {
+    if (status == -1 || !R_FINITE(result)) {
         return R_NaN;
     }
     
+    // Store in cache
+    cache_store_smart(a, b, c, result);
     return result;
 }
